@@ -266,7 +266,7 @@ pub fn assemble<E: UserDefinedError, I: 'static + InstructionMeter>(
         }
     }
     insn_ptr = 0;
-    let mut bpf_functions: BTreeMap<u32, (usize, String)> = BTreeMap::new();
+    let mut bpf_functions = BTreeMap::new();
     resolve_call(&mut bpf_functions, &labels, "entrypoint", None)?;
     let mut instructions: Vec<Insn> = Vec::new();
     for statement in statements.iter() {
@@ -361,8 +361,7 @@ pub fn assemble<E: UserDefinedError, I: 'static + InstructionMeter>(
     }
     let program = instructions
         .iter()
-        .map(|insn| insn.to_vec())
-        .flatten()
+        .flat_map(|insn| insn.to_vec())
         .collect::<Vec<_>>();
     Executable::<E, I>::from_text_bytes(&program, verifier, config, syscall_registry, bpf_functions)
         .map_err(|err| format!("Executable constructor {:?}", err))
