@@ -11,11 +11,13 @@ extern crate solana_rbpf;
 extern crate test_utils;
 extern crate thiserror;
 
+use byteorder::{ByteOrder, LittleEndian};
 #[cfg(all(not(windows), target_arch = "x86_64"))]
 use rand::{rngs::SmallRng, RngCore, SeedableRng};
 use solana_rbpf::{
     assembler::assemble,
-    elf::{ElfError, Executable},
+    ebpf,
+    elf::{register_bpf_function, ElfError, Executable},
     error::EbpfError,
     memory_region::AccessType,
     memory_region::MemoryMapping,
@@ -23,7 +25,7 @@ use solana_rbpf::{
     user_error::UserError,
     vm::{Config, EbpfVm, SyscallObject, SyscallRegistry, TestInstructionMeter},
 };
-use std::{fs::File, io::Read};
+use std::{collections::BTreeMap, fs::File, io::Read};
 use test_utils::{PROG_TCP_PORT_80, TCP_SACK_ASM, TCP_SACK_MATCH, TCP_SACK_NOMATCH};
 
 macro_rules! test_interpreter_and_jit {
@@ -34,7 +36,8 @@ macro_rules! test_interpreter_and_jit {
         $vm.bind_syscall_context_object(Box::new($syscall_context_object), None).unwrap();
     };
     ($executable:expr, $mem:tt, ($($location:expr => $syscall_function:expr; $syscall_context_object:expr),* $(,)?), $check:block, $expected_instruction_count:expr) => {
-        let check_closure = $check;
+        #[allow(unused_mut)]
+        let mut check_closure = $check;
         let (instruction_count_interpreter, _tracer_interpreter) = {
             let mut mem = $mem;
             let mut vm = EbpfVm::new(&$executable, &mut [], &mut mem).unwrap();
@@ -45,7 +48,8 @@ macro_rules! test_interpreter_and_jit {
         };
         #[cfg(all(not(windows), target_arch = "x86_64"))]
         {
-            let check_closure = $check;
+            #[allow(unused_mut)]
+            let mut check_closure = $check;
             let compilation_result = Executable::<UserError, TestInstructionMeter>::jit_compile(&mut $executable);
             let mut mem = $mem;
             let mut vm = EbpfVm::new(&$executable, &mut [], &mut mem).unwrap();
@@ -76,6 +80,15 @@ macro_rules! test_interpreter_and_jit {
 }
 
 macro_rules! test_interpreter_and_jit_asm {
+    ($source:tt, $config:tt, $mem:tt, ($($location:expr => $syscall_function:expr; $syscall_context_object:expr),* $(,)?), $check:block, $expected_instruction_count:expr) => {
+        #[allow(unused_mut)]
+        {
+            let mut syscall_registry = SyscallRegistry::default();
+            $(test_interpreter_and_jit!(register, syscall_registry, $location => $syscall_function; $syscall_context_object);)*
+            let mut executable = assemble($source, None, $config, syscall_registry).unwrap();
+            test_interpreter_and_jit!(executable, $mem, ($($location => $syscall_function; $syscall_context_object),*), $check, $expected_instruction_count);
+        }
+    };
     ($source:tt, $mem:tt, ($($location:expr => $syscall_function:expr; $syscall_context_object:expr),* $(,)?), $check:block, $expected_instruction_count:expr) => {
         #[allow(unused_mut)]
         {
@@ -83,10 +96,7 @@ macro_rules! test_interpreter_and_jit_asm {
                 enable_instruction_tracing: true,
                 ..Config::default()
             };
-            let mut syscall_registry = SyscallRegistry::default();
-            $(test_interpreter_and_jit!(register, syscall_registry, $location => $syscall_function; $syscall_context_object);)*
-            let mut executable = assemble($source, None, config, syscall_registry).unwrap();
-            test_interpreter_and_jit!(executable, $mem, ($($location => $syscall_function; $syscall_context_object),*), $check, $expected_instruction_count);
+            test_interpreter_and_jit_asm!($source, config, $mem, ($($location => $syscall_function; $syscall_context_object),*), $check, $expected_instruction_count);
         }
     };
 }
@@ -2270,6 +2280,44 @@ fn test_err_stack_out_of_bound() {
     );
 }
 
+#[test]
+fn test_err_mem_access_out_of_bound() {
+    let mem = [0; 512];
+    let mut prog = [0; 32];
+    prog[0] = ebpf::LD_DW_IMM;
+    prog[16] = ebpf::ST_B_IMM;
+    prog[24] = ebpf::EXIT;
+    for address in [0x2u64, 0x8002u64, 0x80000002u64, 0x8000000000000002u64] {
+        LittleEndian::write_u32(&mut prog[4..], address as u32);
+        LittleEndian::write_u32(&mut prog[12..], (address >> 32) as u32);
+        let mut bpf_functions = BTreeMap::new();
+        register_bpf_function(&mut bpf_functions, 0, "entrypoint", true).unwrap();
+        #[allow(unused_mut)]
+        let mut executable = Executable::<UserError, TestInstructionMeter>::from_text_bytes(
+            &prog,
+            None,
+            Config::default(),
+            SyscallRegistry::default(),
+            bpf_functions,
+        )
+        .unwrap();
+        test_interpreter_and_jit!(
+            executable,
+            mem,
+            (),
+            {
+                |_vm, res: Result| {
+                    matches!(res.unwrap_err(),
+                        EbpfError::AccessViolation(pc, access_type, vm_addr, len, name)
+                        if access_type == AccessType::Store && pc == 31 && vm_addr == address && len == 1 && name == "unknown"
+                    )
+                }
+            },
+            2
+        );
+    }
+}
+
 // CALL_IMM & CALL_REG : Procedure Calls
 
 #[test]
@@ -2721,7 +2769,7 @@ impl SyscallObject<UserError> for NestedVmSyscall {
             syscall_registry
                 .register_syscall_by_name::<UserError, _>(b"NestedVmSyscall", NestedVmSyscall::call)
                 .unwrap();
-            let mem = &mut [depth as u8 - 1, throw as u8];
+            let mem = [depth as u8 - 1, throw as u8];
             let mut executable = assemble::<UserError, TestInstructionMeter>(
                 "
                 ldabsb 0
@@ -2735,27 +2783,7 @@ impl SyscallObject<UserError> for NestedVmSyscall {
                 syscall_registry,
             )
             .unwrap();
-            #[cfg(all(not(windows), target_arch = "x86_64"))]
-            {
-                Executable::<UserError, TestInstructionMeter>::jit_compile(&mut executable)
-                    .unwrap();
-            }
-            let mut vm = EbpfVm::new(&executable, &mut [], mem).unwrap();
-            vm.bind_syscall_context_object(Box::new(NestedVmSyscall {}), None)
-                .unwrap();
-            let mut instruction_meter = TestInstructionMeter { remaining: 6 };
-            #[cfg(any(windows, not(target_arch = "x86_64")))]
-            {
-                *result = vm.execute_program_interpreted(&mut instruction_meter);
-            }
-            #[cfg(all(not(windows), target_arch = "x86_64"))]
-            {
-                *result = vm.execute_program_jit(&mut instruction_meter);
-            }
-            assert_eq!(
-                vm.get_total_instruction_count(),
-                if throw == 0 { 6 } else { 5 }
-            );
+            test_interpreter_and_jit!(executable, mem, (b"NestedVmSyscall" => NestedVmSyscall::call; NestedVmSyscall {}), { |_vm, res: Result| { *result = res; true } }, if throw == 0 { 6 } else { 5 });
         } else {
             *result = if throw == 0 {
                 Ok(42)
@@ -3005,8 +3033,8 @@ fn test_non_terminate_early() {
         {
             |_vm, res: Result| {
                 matches!(res.unwrap_err(),
-                    EbpfError::ElfError(ElfError::UnresolvedSymbol(a, b, c))
-                    if a == "Unknown" && b == 35 && c == 48
+                    EbpfError::UnsupportedInstruction(pc)
+                    if pc == 35
                 )
             }
         },
@@ -3157,7 +3185,7 @@ fn test_err_call_unresolved() {
         [],
         (),
         {
-            |_vm, res: Result| matches!(res.unwrap_err(), EbpfError::ElfError(ElfError::UnresolvedSymbol(symbol, pc, offset)) if symbol == "Unknown" && pc == 34 && offset == 40)
+            |_vm, res: Result| matches!(res.unwrap_err(), EbpfError::UnsupportedInstruction(pc) if pc == 34)
         },
         6
     );
@@ -3171,7 +3199,7 @@ fn test_err_unresolved_elf() {
     let mut elf = Vec::new();
     file.read_to_end(&mut elf).unwrap();
     let config = Config {
-        reject_unresolved_syscalls: true,
+        reject_broken_elfs: true,
         ..Config::default()
     };
     assert!(
@@ -3391,16 +3419,13 @@ fn test_tcp_sack_nomatch() {
 
 #[cfg(all(not(windows), target_arch = "x86_64"))]
 fn execute_generated_program(prog: &[u8]) -> bool {
-    use solana_rbpf::{elf::register_bpf_function, verifier::check};
-    use std::collections::BTreeMap;
-
     let max_instruction_count = 1024;
     let mem_size = 1024 * 1024;
     let mut bpf_functions = BTreeMap::new();
-    register_bpf_function(&mut bpf_functions, 0, "entrypoint", false).unwrap();
+    register_bpf_function(&mut bpf_functions, 0, "entrypoint", true).unwrap();
     let executable = Executable::<UserError, TestInstructionMeter>::from_text_bytes(
         prog,
-        Some(check),
+        Some(solana_rbpf::verifier::check),
         Config {
             enable_instruction_tracing: true,
             ..Config::default()
@@ -3458,7 +3483,6 @@ fn execute_generated_program(prog: &[u8]) -> bool {
 #[cfg(all(not(windows), target_arch = "x86_64"))]
 #[test]
 fn test_total_chaos() {
-    use solana_rbpf::ebpf;
     let instruction_count = 6;
     let iteration_count = 1000000;
     let mut program = vec![0; instruction_count * ebpf::INSN_SIZE];
